# -*- coding: utf-8 -*-

# MIT License
#
# Copyright (c) 2021 Mehmet Umut Mutlu, Nazlıcan Yetimaslan, İlker Atagun
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

loans = {
    "alaka": [["a\u02D0", 2]],
    "hakim": [["\u0251\u02D0", 1]],
    "memur": [["e\u02D0", 1]],
    "temin": [["e\u02D0", 2]],
    "tecil": [["e\u02D0", 2]],
    "tesir": [["e\u02D0", 2]],
    "itibaren": [["i\u02D0", 0]],
    "ilan": [["i\u02D0", 0], ["a", 2]],
    "lider": [["i\u02D0", 1]],
    "muaf": [["u\u02D0", 1]],
    "ödlek": [["t", 1]],
    "tahin": [["\u0251\u02D0", 2]],
    "nisan": [["i\u02D0", 1]],
    "niçin": [["i\u02D0", 1]],
    "rakım": [["\u0251\u02D0", 1]],
    "lüfer": [["y\u02D0", 1]],
    "kağıt": [["c", 0], ["a", 2]],
    "lamba": [["a", 1]],
    "kip": [["k", 0]]
    }

# loans2 = {
#     "alaka": "\u0251\u0142a\u02D0k\u0251",
#     "hakim": "x\u0251\u02D0cIm",
#     "memur": "me\u02D0mU\u0263",
#     "temin": "t\u02B0e\u02D0mIn",
#     "tecil": "t\u02B0e\u02D0d\u0292Il",
#     "tesir": "t\u02B0e\u02D0sI\u0263",
#     "itibaren": "i\u02D0tIb\u0251r\u025Bn",
#     "ilan": "i\u02D0\u0142an",
#     "lider": "li\u02D0d\u025B\u0263",
#     "muaf": "mu\u02D0\u0251f",
#     "ödlek": "\u0153tl\u025Bc",
#     "tahin": "t\u02B0\u0251\u02D0çIn",
#     "nisan": "ni\u02D0s\u0251n",
#     "niçin": "ni\u02D0t\u0283In",
#     "rakım": "r\u0251\u02D0k\u0268m",
#     "lüfer": "ly\u02D0f\u025B\u0263"
#     }